# File generated from our OpenAPI spec by Stainless.

from typing import Optional
from typing_extensions import Literal

__all__ = ["PaymentOrderSubtype"]

PaymentOrderSubtype = Optional[Literal["CCD", "CIE", "CTX", "IAT", "PPD", "TEL", "WEB"]]
