# File generated from our OpenAPI spec by Stainless.

from __future__ import annotations

from ..._models import BaseModel

__all__ = ["AsyncResponse"]


class AsyncResponse(BaseModel):
    id: str

    object: str
